/**
 * @file        ADSO-9549
 * @description Create time-based unit and verify able to edit efficiency via Capacities view
 * @author      Chan Clarence (ccn7@3ds.com)
 * @copyright   Dassault Systemes
 */
import { qCustomMatcher } from '../../e2elib/lib/src/main/qmatchers.const';
import { AppMP, Demo, Scenario } from '../../libmp/appmp';
import { DataUnitTimeCapacityInfo, DataUnitCapacityType } from '../../libmp/data/data.unit';
import { DataFoodBaseEntityName } from '../../libmp/data/data.navigation';
import { DialogUnit } from '../../libmp/dialogs/dialog.unit';
import { DialogTimeCapacity } from '../../libmp/dialogs/dialog.timecapacity';
import { MatrixEditorTimeCapacitiesContextMenuItem } from '../../libmp/forms/form.timecapacities';
import { ListEntityContextMenuItem } from '../../libmp/forms/form.unit';
import { startOfPlanningYear } from '../../libmp/data/data.period';

describe('ADSO-9549 - Create time-based unit and verify able to edit efficiency via Capacities view', () => {
  const appMP = AppMP.getInstance();
  let dlgUnit: DialogUnit;
  let dlgTimeCapacity: DialogTimeCapacity;
  const listEntity = appMP.viewEntity.formEntity.listEntity;
  const meTimeCapacities = appMP.viewCapacitiesTime.formTimeCapacities.meTimeCapacities;
  const europe = DataFoodBaseEntityName.Europe;
  const newUnitName = 'A time unit';
  const capacityDate = `1-Feb-${startOfPlanningYear}`;
  const oldEfficiency = '1';
  const newEfficiency = '2';
  const newEfficiencyWithDecimal = `${newEfficiency}.00`;

  beforeAll(async () => {
    jasmine.addMatchers(qCustomMatcher);
    await appMP.login();
  });

  afterAll(async () => {
    await appMP.viewEntity.reset();
    await appMP.viewCapacitiesTime.reset();
    await appMP.cleanupAndLogout();
  });

  afterEach(async () => {
    await appMP.checkToastMessage();
  });

  it(`Setup - ${AppMP.getDemoDataPath(Demo.Food, Scenario.Base)}`, async () => {
    await appMP.createDemoDataset(Demo.Food, Scenario.Base, false);
  });

  it(`Setup - Open view ${appMP.viewEntity.viewPath}.`, async () => {
    await appMP.viewEntity.switchTo();
  });

  it('Step 1 - In list "Units and Stocking Points", right click and select menu "Create unit".', async () => {
    [dlgUnit] = await listEntity.selectContextMenu(ListEntityContextMenuItem.CreateUnit);
  });

  it(`Step 2 - In dialog, set Parent = "${europe}", Name = "${newUnitName}" and Capacity type = "${DataUnitCapacityType.Time}". Click OK to create unit.`, async () => {
    await dlgUnit.updateDialogValues({Parent: europe, Name: newUnitName, CapacityType: DataUnitCapacityType.Time});
    await dlgUnit.clickOK();
  });

  it(`Step 3 - Verify unit created in list by inspecting Name = "${newUnitName}" and Capacity Type = "${DataUnitCapacityType.Time}".`, async () => {
    await listEntity.verifyRowExists({Name: newUnitName, 'Capacity type': DataUnitCapacityType.Time}, [{Name: europe}]);
  });

  it(`Step 4 - Open view ${appMP.viewCapacitiesTime.viewPath}.`, async () => {
    await appMP.viewCapacitiesTime.switchTo();
  });

  it(`Step 5 - In matrix "Time Capacities", right click cell for row = "${newUnitName} > ${DataUnitTimeCapacityInfo.Efficiency}" and column "${capacityDate}". Select menu "Edit".`, async () => {
    [dlgTimeCapacity] = await meTimeCapacities.rightClickCellSelectContextmenu(MatrixEditorTimeCapacitiesContextMenuItem.Edit, newUnitName, DataUnitTimeCapacityInfo.Efficiency, capacityDate);
  });

  it(`Step 6 - In dialog, verify ${DataUnitTimeCapacityInfo.Efficiency} = ${oldEfficiency}.
      Update to be ${newEfficiency} and click OK. Verify matrix cell for ${DataUnitTimeCapacityInfo.Efficiency} updated to ${newEfficiencyWithDecimal}.`, async () => {
    await dlgTimeCapacity.verifyDialogValues({Efficiency: oldEfficiency});
    await dlgTimeCapacity.updateDialogValues({Efficiency: newEfficiency});
    await dlgTimeCapacity.clickOK();

    await meTimeCapacities.verifyCellValue(newUnitName, DataUnitTimeCapacityInfo.Efficiency, capacityDate, newEfficiencyWithDecimal);
  });
});
